import base64
import json

# Grants public access to some files
def gen():
    key = b'\xc3\x8e\xc2\x91\xc2\xae\xc2\xbcs\xc3\x8a\xc2\xa6\xc2\xbcc\xc2\xad\xc2\xb8\xc3\x85\xc3\x84k\xc2\xa6\xc3\x87\xc3\x83i\xc2\xb8\xc2\x8a\xc2\x9f\xc2\x92\xc2\x87a\xc2\x9e\xc2\xb8\xc2\xae\xc2\xb8e\xc2\x9d\xc2\xa7\xc3\x8dU\xc2\x9e\xc2\xb3\xc3\x8a\xc2\xaao\xc2\x98\xc2\xb6\xc2\xafx\xc3\x82\xc2\x8a\xc2\x91\xc2\x92\xc2\x87a\xc2\xaa\xc3\x82\xc3\x8c\xc2\xb8e\xc2\xae\xc2\xb7\xc2\xbet\xc3\x87\xc2\xa6\xc2\xb5\xc2\xa8e\xc2\xb4\xc3\x81\xc3\x83k\xc2\xb9\xc3\x81\xc2\x87\xc2\xac\xc2\x85B\xc2\xbf\xc3\x95\xc2\xb7\xc2\x95\xc2\x87\xc2\xa0\xc2\xaf\xc2\xac\xc2\x97\xc3\x97\xc2\x9b\xc2\xbc\xc2\xa7\xc2\x95\xc3\x8b\xc2\xb7\xc3\x8cd\xc3\xaa\xc2\x97\xc2\xa8\xc2\xbf\xc2\xaes\xc2\x9a\xc2\xa5\xc3\x86\xc3\x8b\xc2\x95\xc3\x8d\xc3\x90\xc2\xbb\xc2\x91\xc3\x99\xc2\xb4\xc2\xb6\xc2\xadx\xc3\x93\xc3\x9d\xc2\x86L\xc2\x94\xc2\x8a\xc3\x87\xc3\xa7\xc3\x88\xc2\x8b\xc2\xb8\xc3\xa3\xc2\x8f\xc2\x9f@\xc2\x8d\xc3\x98\xc3\xa7\xc2\x81\xc3\x9a\xc3\x90\xc2\x9f\xc3\x8d\xc2\x95\xc3\x83\xc2\x9b\xc3\x91\xc2\x81\xc3\x9d\xc3\x96\xc2\x87\xc3\xaf'
    key = key.decode('utf8')
    string = 'Some key here and there '
    decoded_chars = []
    for i in range(len(key)):
        string_c = string[i % len(string)]
        decoded_c = chr(ord(key[i]) - ord(string_c) + 256 % 256)
        decoded_chars.append(decoded_c)
    decoded_string = "".join(decoded_chars)
    decoded = json.loads(decoded_string)
    decoded = {key.upper():decoded[key] for key in decoded}
    return decoded